#!/usr/bin/env ruby

raise "Not a standalone command" if __FILE__ == $0

module TeXDist
    class Application
        class Bridge
            include Comparable
            attr_accessor :volume,:domain,:base_path,:path,:principal
            attr_accessor :_archs,:texbin,:preferred_bin,:user_domain,:_version,:main
            @@hardware=`uname -m`.chomp
            @@architecture=`uname -p`.chomp
            @@archs_by_hard={"i386"=> %w{i386-darwin i386 x86-darwin x86 universal-darwin universal ppc-darwin ppc powerpc-darwin powerpc},
            "ppc"=> %w{ppc-darwin ppc powerpc-darwin powerpc universal-darwin universal},
            "x86_64"=> %w{x86_64-darwin x86_64 universal_64-darwin universal_64 i386-darwin i386 x86-darwin x86 universal-darwin universal ppc-darwin ppc powerpc-darwin powerpc},
            "ppc64"=> %w{ppc64-darwin ppc64 powerpc64-darwin powerpc64 universal_64-darwin ppc-darwin ppc powerpc-darwin powerpc universal-darwin universal}
            }[@@hardware]
            def Bridge.nouveau(path,with_ext=true,verify=true)
                if verify and not path.exist?
                    my_log("No TeX distribution bridge at #{path}.",3)
                elsif with_ext and not path.my_match_ext?
                    my_log("No TeX distribution bridge (Bad file extension) at #{path}.",3)
                elsif path.my_version.exist?
                    return new(path,with_ext)
                else
                    my_log("No TeX distribution bridge (Missing version) at #{path}.",3)
                end
                nil
            end
            def initialize(path,with_ext)
                @volume,@domain,@base_path=path.my_vodoba
                @path=path
                my_log("Possible TeX distribution bridge found at #{path}.",1)
                @main= true
                @user_domain=false
                t=programs_p.my_texbin
                if t.symlink? and (t+'tex').executable?
                    my_log("Usable TeX distribution bridge found at #{path}.",2)
                    @preferred_bin=t.readlink.to_s
                    return
                end
                if (programs_p+@@hardware+'tex').executable?
                    @preferred_bin=@@hardware
                else
                    @preferred_bin=available_archs.first
                end
                repair
            end
            def <=>(b)
                if has_arch? and not b.has_arch?
                    return -1
                elsif not has_arch? and b.has_arch?
                    return 1
                end
                out=(volume.to_s <=> b.volume.to_s)
                return out if 0!=out
                out=(domain.to_s <=> b.domain.to_s)
                return out if 0!=out
                out=(base_path.to_s <=> b.base_path.to_s)
                return out if 0==out
                lhs=corename_p.to_s
                rhs=b.corename_p.to_s
                return -1 if rhs.start_with?(lhs)
                return  1 if lhs.start_with?(rhs)
                out
            end
            def version
                @_version.nil?? @_version=path.my_version.read.to_i: @_version
            end
            def version=(v)
                path.my_version.open('w') { |f| f.write(v.to_s) }
                @_version = nil
            end
            def has_arch?
                available_archs.count>0
            end
            def repair
                return false if version<Helper.version
                if preferred_bin.nil?
                    my_log("No usable TeX distribution bridge at #{path}.",2)
                    return false
                else
                    my_log("Usable TeX distribution bridge found at #{path}.",2)
                end
                t=programs_p.my_texbin
                return true if t.symlink? and (t+'tex').executable?
                t.delete if t.exist? or t.symlink?
                t.make_symlink(bin) unless bin.nil?
                true
            rescue StandardError => e
                my_log(e,'To repair global bridges, you might need administrator privileges using sudo.')
                false
            end
            def available_archs
                if @_archs.nil?
                    @_archs=[]
                    ps=[]
                    @@archs_by_hard.each do |a|
                        if (p=programs_p+a+'tex').executable? and not ps.index(p=p.realpath)
                            @_archs<<a
                            ps<<p
                        end
                    end
                end
                @_archs
            end
            def active_architecture
                if texbin_p.symlink?
                    p=texbin_p.readlink
                    return p if available_archs.index(p.to_s)
                end
                nil
            end
            def basename_p
                path.basename
            end
            def corename_p
                path.my_corename
            end
            def contents_p
                path.my_contents
            end
            def contents_target_p
                contents_p.symlink?? path+contents_p.readlink: contents_p
            end
            def base_p(realpath=false)
                realpath ? contents_p.my_base.my_realpath: contents_p.my_base
            end
            def root_p(realpath=false)
                realpath ? contents_p.my_root.my_realpath: contents_p.my_root
            end
            def programs_p
                contents_p.my_programs
            end
            def texbin_p(realpath=false)
                @texbin={} if @texbin.nil?
                if @texbin[realpath].nil?
                    r = programs_p.my_texbin
                    @texbin[realpath]=realpath ? (r.exist? ? r.my_realpath: nil): r
                end
                return @texbin[realpath]
            end
            def fix_texbin(simul8=false)
                if not active_architecture and @preferred_bin
                    texbin_p.my_make_symlink(@preferred_bin,simul8)
                end
            end
            def activate_architecture(architecture,simul8=false)
                a = architecture=='default' ? available_archs.first.to_s: architecture
                if not a.nil? and not a.empty?
                    p=Pathname.new(a)
                    if not texbin_p.nil? and available_archs.index(a)
                        texbin_p.my_make_symlink(p,simul8)
                    else
                        my_error "Cannot activate architecture #{p}"
                    end
                else
                    my_error "No architecture to activate"
                end
            end
        end
    end
end
